import pandas as pd

from spotlight.common.protoSelector import ProtoABSSelector
from spotlight.save.save import Saver
from spotlight.common.ErrRetry import ErrRetryF
import spotlight.common.myFileDialog as myfd

class SaverPart(ProtoABSSelector):

    ref:str
    refLi:list
    cName:str
    dfTmp:pd.DataFrame

    def run(self):
        self.cName = self.selectColumn("기준컬럼 선택")

        msg = "1. 1개 기준으로 추출 (특정 계정과목만 추출하는 경우 등 )\n"
        msg += "2. 다중 기준으로 추출 (다수 계정과목을 추출하는 경우 등) : 직접 입력\n"
        msg += "3. 다중 기준으로 추출 (다수 계정과목을 추출하는 경우 등) : 엑셀로 Import\n"
        print(msg)
        flag = input(">>")
        match(flag):
            case '1': self.dfTmp = self._single()
            case '2': self.dfTmp = self._multi1()
            case '3': self.dfTmp = self._multi2()
            case _: print("잘못 입력");return
        
        print("추출결과>> ")
        print(str(self.dfTmp.shape[0])+ "행")
        print(self.dfTmp.head(10))
        print("저장합니다...")
        Saver(self.dfTmp).run()
    
    @ErrRetryF
    def _single(self) -> pd.DataFrame:
        print("단일 기준으로 추출합니다. i.e. 특정 1개 계정과목 원장만 추출")
        self.ref = self._setValue()        
        return self.df[self.df[self.cName] == self.ref]

    @ErrRetryF
    def _multi1(self) -> pd.DataFrame:
        print("복수 기준으로 추출합니다. i.e. 특정 계정과목들에 해당하는 원장만 추출 : 직접 입력")        
        self.refLi = self._addList()
        return self.df.loc[self.df[self.cName].isin(self.refLi)]

    @ErrRetryF
    def _multi2(self) -> pd.DataFrame:
        print("복수 기준으로 추출합니다. i.e. 특정 계정과목들에 해당하는 원장만 추출 : 엑셀로 입력받음")        
        dfTmp = pd.read_excel(myfd.askopenfilename("Select excel file"))        
        cNameTgt = self.selectColumn("엑셀에서 추출한 내역 중 필요한 컬럼 선택", dfTmp)
        self.refLi = dfTmp[cNameTgt].drop_duplicates().to_list() #중복을 제거함
        if input("String으로 변환하려면 Y>>") == 'Y': self.refLi = list(map(str,self.refLi))
        print(self.refLi)
        return self.df.loc[self.df[self.cName].isin(self.refLi)]

    def _addList(self) -> list:
        refLi:list = []
        print("입력을 받습니다..  (종료하려면 q)")
        while True:
            refTmp = self._setValue()
            if refTmp == 'q': return refLi            
            refLi.append(refTmp)
            print("현재 목록:")
            print(refLi)

    def _setValue(self) -> str | int:
        refTmp = input("추출할 값을 입력하세요(문자열)>>")
        if input("문자열로 적용하려면 1, 숫자로 형변환하려면 2(int로 형변환)>>") == '2': refTmp = int(refTmp)
        return refTmp






import numpy as np
import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class AutoManual:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df

    @ErrRetryF
    def run(self, cNameBase:str, cNameAM:str): 

        strFlag = input("자동전표 기준으로 포함할 문자열을 입력하세요(복수는 Pipe 사용) (ex1. Auto) (ex2. Auto|System)  >>") or 'Auto'
        strA = input("자동전표인 경우 입력할 문자열(기본값 A)>>") or 'A'
        strM = input("기타 수동전표인 경우 입력할 문자열(기본값 M)>>") or 'M'        

        self.df[cNameAM] = np.where( self.df[cNameBase].str.contains(strFlag) , strA, strM)

        print("DONE")
        



#RECON 파일 추출
import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF
from spotlight.modify.modify import Modifier

class ReconGL(Modifier): #Inherit Modifier to use 'selectColumn'

    @ErrRetryF    
    def run(self):
        cNameAcct = self.selectColumn("Select Account Number (to group by)")
        cNameAmt = self.selectColumn("Select Amount LC (to sum)")
        self.export(cNameAcct, cNameAmt)
    
    def export(self, cNameAcct:str, cNameAmt:str): #cName1 : 변경할 컬럼, #cName2 : 차대컬럼

        #먼저 float64로 바꾼다
        self.df[cNameAmt] = self.df[cNameAmt].astype('float64')

        #Run
        fileName = input("검증결과 추출할 파일명을 입력하세요(기본값 GL_RECON.xlsx)>>") or 'GL_RECON.xlsx'
        
        self.df.groupby(cNameAcct)[cNameAmt].sum().to_excel(fileName)
        
        print("DONE")
        




from spotlight.concatText.concatText import ConcatText
from spotlight.common.ErrRetry import ErrRetryF
# 함수부

class ConcatTextTest(ConcatText): #INHERIT

    def run(cls): #OVERIDE
        cls.getFiles("헤더 테스트할 텍스트파일이 있는 폴더 지정")
        #cls.filenameNew = input("new file name(result.txt)>>") or 'result.txt'                
        cls.encodingOld = input("encodingOld(cp949)>>") or 'cp949'
        #cls.encodingNew = input("encodingNew(utf8)>>") or 'utf8'

        # flag = input("헤더가 포함되어 있습니까?(기본값 Y). Y인 경우 첫번째 파일만 포함함)>>") or 'Y'
        # if flag == 'Y': cls.bHeader = True
        # else: cls.bHeader = False
        cls.concat()
        print("DONE")

    @ErrRetryF
    def concat(cls, bHeader:bool = None): #OVERRIDE        
        print("총",len(cls.liTgt),"개 파일에 대해 헤더 순환출력")        

        #파일로 출력하도록 변경 : fileResult
        fileResultName = 'ColumnTestResult.txt'
        fileResult = open(fileResultName, mode='tw', encoding='utf8')

        for fileName in cls.liTgt:                                              

            fileResult.write(fileName)
            fileResult.write('\n')

            fileNow = open(fileName, 'rt', encoding=cls.encodingOld)           

            fileResult.write(fileNow.readline()) #여기는 \n이 포함됨?

            fileNow.close()

        fileResult.close()





# 231002. 영국 요청에 따라 특정 전표번호 식별을 위해 임시테이블 추가
from mylib import txt2df2db as mytdd
from mylib import myFileDialog as myfd
path = myfd.askopenfilename()

import pandas as pd
df= pd.read_excel(path)

mytdd.df2db(df,'frl','tmp_ami') #myLib 활용




import numpy as np
import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class ApplyDC:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame): self.df = df

    #차대구분에 따라 (-)처리 : 변경컬럼선택 / 차대컬럼선택 / 차변구분자 선택 / 시행
    @ErrRetryF    
    def run(self, cName1:str, cName2:str): #cName1 : 변경할 컬럼, #cName2 : 차대컬럼

        #먼저 float64로 바꾼다
        self.df[cName1] = self.df[cName1].astype('float64')

        #Run
        IndicatorDebit = input("Indicator의 차변구분자를 입력하세요(기본값 'D')>>") or 'D'        
        self.df[cName1] = np.where(self.df[cName2] == IndicatorDebit
                 , self.df[cName1]*1
                 , self.df[cName1]*(-1))
        
        print("DONE")
        



import os
import pandas as pd
import spotlight.common.myFileDialog as myfd

class ReadColumnLength:
    def run(self):
        path = myfd.askopenfilename("Select a sample GL file")
        encoding = input("encoding, 기본값 cp949>>") or 'cp949'
        sep = input("sep, 기본값 tsv>>") or '\t'
        df = pd.read_csv(path, encoding=encoding, sep=sep, low_memory=False)
        
        dfTmp = pd.DataFrame()
        for column in df:
            print(column,"->", df[column].astype(str).str.len().max())
            #di[column] = df[column].astype(str).str.len().max()

            di = {'ColumnName':column,
                  'Length':df[column].astype(str).str.len().max()
                }   

            dfDi = pd.DataFrame(di, index=[0])
            dfTmp = pd.concat([dfTmp, dfDi])
        
        dfTmp.to_excel("ColumnLength.xlsx", index=False)
        print("ColumnLength.xlsx 추출완료. Table 설계하세요")        

def runReadColumnLength():
    ReadColumnLength().run()

if __name__=='__main__':
    runReadColumnLength()
    
    






# 추가로 제공한 4월 원장 가공. by Excel
# 엑셀을 데이터프레임으로 합친다.

# 선언부
import os
import glob
import myFileDialog as myFD
import tqdm
import pandas as pd

#1) 대상파일을 읽는다.
path = myFD.askdirectory()
listFiles = glob.glob(path + '/*.xlsx')    

pbar = tqdm.tqdm(total=len(listFiles), desc="개시")
dfNew = pd.DataFrame()
countRecord = 0

for item in listFiles:
    pbar.desc = item
    df = pd.read_excel(item)
    countRecord += df.shape[0]
    dfNew = pd.concat([dfNew,df])
    tmp = pbar.update(1)

pbar.close()





import numpy as np
import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class CreateUnsigned:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame): self.df = df

    #차대구분에 따라 (-)처리 : 변경컬럼선택 / 차대컬럼선택 / 차변구분자 선택 / 시행
    @ErrRetryF    
    def run(self, cNameSigned:str, cNameUnsignedD:str, cNameUnsignedC:str): 

        #먼저 float64로 바꾼다
        self.df[cNameSigned] = self.df[cNameSigned].astype('float64')

        #Run        
        self.df[cNameUnsignedD] = np.where(self.df[cNameSigned] >= 0
                 , self.df[cNameSigned]
                 , 0)
        
        self.df[cNameUnsignedC] = np.where(self.df[cNameSigned] < 0
                 , self.df[cNameSigned] * -1
                 , 0)

        print("DONE")
        



import glob

import pandas as pd
import tqdm
import sqlalchemy

from spotlight.common import myFileDialog as myfd
from spotlight.txt2db.mod.dbcon import DbCon
from spotlight.txt2db.mod.txt2df import Txt2Df
from spotlight.txt2db.mod.df2db import Df2Db

class Txt2Db:

    path : str
    listFiles : list

    objEngine : sqlalchemy.Engine

    def run(cls):                
        cls.path = myfd.askdirectory() #폴더를 input 받는다.        
        cls.checkFiles() #폴더 내 텍스트 배열을 인식한다.   
        cls.importAndInsert() #파일을 순환 : import / insert

    def checkFiles(cls):
        ext = input("확장자(기본값, tsv)>>") or 'tsv'
        cls.listFiles = glob.glob(cls.path+'/*.'+ext)
        print(cls.listFiles)
    
    def importAndInsert(cls):
        cls.objEngine = DbCon().connect() #DB 연결

        pbar = tqdm.tqdm(total=len(cls.listFiles))
        objT2D = Txt2Df() #create object to import textfile
        objD2D = Df2Db(cls.objEngine)

        i = 0

        for file in cls.listFiles: #파일을 순환하면서,
            pbar.set_description(file)

            df = objT2D.run(file) #IMPORT
            objD2D.insert(df, 'lse') #and insert
            
            pbar.update(1)          
            # i += 1
            # if i == 1: print("DEBUG END"); break
        
        pbar.close()

def runTxt2Db(): #CALLER
    Txt2Db().run()



#전역부
from mylib import txt2df2db as mytdd
from mylib import myFileDialog as myfd
import os
import glob
import tqdm
import sys

def batch():
    
    path = myfd.askdirectory()
    listFiles = glob.glob(path + '/*.txt')

    pbar = tqdm.tqdm(total=len(listFiles) , desc="Files 순환")    

    for i in listFiles:
        pbar.desc = i

        print("Start:")        
        fileName = i
        chunksize = int(sys.argv[1])
        df = mytdd.txt2df(fileName, "\t", True, chunksize)
        mytdd.df2db(df, 'frl', 'frl_new', chunksize) #두번째 인자 : 연결할 데이터베이스
        print("End:")

        pbar.update(1)
    pbar.close()

#MAIN
if(__name__ == "__main__"):
    mytdd.getArgv1() #매개변수 없으면 추가
    print("순환을 개시합니다.")
    batch()

    


#
# File Dialog 호출하여 Path를 반환한다.
#
# 231019. Focus 받도록 수정

import tkinter as tk
import tkinter.filedialog as fd
import os

def askopenfilename(title:str="열기") -> str :
    root = tk.Tk()
    root.attributes('-topmost', True)
    root.withdraw()
    path = fd.askopenfilename(
        parent=root,
        initialdir=os.getcwd(),
        title=title)
    root.destroy()
    return path

def askdirectory(title:str="열기") -> str :
    root = tk.Tk()
    root.attributes('-topmost', True)
    root.withdraw()
    path = fd.askdirectory(
        parent=root,
        initialdir=os.getcwd(),
        title=title)
    root.destroy()
    return path    




import pandas as pd
import pymysql
import sqlalchemy
pymysql.install_as_MySQLdb()
import MySQLdb    
import pyodbc

import spotlight.common.myFileDialog as myfd

class DbCon:

    def connect(cls, flag:str = 'mysql', dbName : str = 'spotlight') -> sqlalchemy.Engine:

        mySQL_ID = input("ID? (DBMS)>>")
        mySQL_PW = input("Password? (DBMS)>>")

        if flag=="mysql":
            #mySQL_ID = ""
            #mySQL_PW = ""
            #mySQL_DB = 'spotlight'
            mySQL_DB = dbName
            #MySQL에 연결하는 경우
            engine = sqlalchemy.create_engine("mysql+mysqldb://"+mySQL_ID+":"+mySQL_PW+"@127.0.0.1/"+mySQL_DB+"?charset=utf8")

        elif flag=="mssql":
            #MSSQL에 연결하는 경우
            engine = sqlalchemy.create_engine("mssql+pyodbc://SA:qkrguddnjs1!@mymssql")
            #engine.connect()
        
        engine.connect()
        print("DB에 연결되었습니다.")
        return engine #engine을 반환 




import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

#정규식 추가할 수 있도록 변경 (정규식 before, after 받음)
'''
콤마 없애기 : '[,]',''
공백 없애기 : '[ ]',''
마지막 - 없애기 : '[-]$',''
'''
class Replacer:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df

    @ErrRetryF
    def run(self, cName:str) -> None: #콤마 없애기

        self.df[cName] = self.df[cName].astype('string') #DEBUG : STRING
        print("string type으로 변경함")

        msg = "1. Kill Comma\n"
        msg += "2. Kill Space\n"
        msg += "9. User-defined Regular expression\n"
        print(msg)
        flag = input(">>")
        
        strBefore:str;strAfter:str
        
        match(flag):
            case '1': strBefore = '[,]' ; strAfter = ''
            case '2': strBefore = '[ ]' ; strAfter = ''
            case '9':
                strBefore = input("변경전 정규식을 직접 입력하세요. (i.e. [-]$) (기본값 : '[ ]')>>") or '[ ]'
                strAfter = input("변경후 정규식을 직접 입력하세요. (기본값 : '')>>") or ''
            case _:
                print("잘못 입력하였습니다."); return

        self._doReplace(cName,strBefore,strAfter)
        print("DONE")

    def _doReplace(self, cName:str, strBefore:str, strAfter:str) -> None:
        self.df[cName] = self.df[cName].replace(strBefore,strAfter,regex=True)



import pandas as pd

from spotlight.read import runReadColumnLength
from spotlight.txt2db import runTxt2Db
from spotlight.concatText import runConcatText
from spotlight.concatText import runConcatText
from spotlight.concatText.concatTextTest import ConcatTextTest
from spotlight.import2df.import2df import runImport2Df
from spotlight.save.save import Saver
from spotlight.save.savePart import SaverPart
from spotlight.automap.automap import AutoMap
from spotlight.modify.modify import Modifier
from spotlight.recon.recon import ReconGL
from spotlight.merge.merge import Merger
from spotlight.common.colors import Colors
from spotlight.common.protoSelector import ProtoABSSelector

class Spotlight(ProtoABSSelector):
    df:pd.DataFrame    
    def run(self):
        text =  "#"*10+"\n"
        text += Colors.RED + "\nPreprocessing\n" + Colors.END
        text += "11. Excel to Text\n"    
        text += "12. concatenate text\n"    
        text += "13. check text header (TEST)\n"    
        text += "14. Merge Text (i.e. BSEG+BKPF) (if already set df, automatically transferred to dfA)\n"    

        text += Colors.RED + "\nUSE SQL\n" + Colors.END
        text += "21. To Insert to SQL, Read columns'length\n"
        text += "22. Create Table => with mySQL\n"
        text += "23. Import file and Insert to DB\n"

        text += Colors.RED + "\nMain Run\n" + Colors.END
        text += "31. Read text to dataframe\n"
        text += "32. Auto_MAP\n"
        text += "33. Modify mode(After Auto_MAP)\n" #조정자는 별도 클래스로 분리 #아직 미구현        
        text += "34. To recon G/L and T/B, export SUM(AMT LC)groupby Acct\n" #조정자는 별도 클래스로 분리 #아직 미구현        

        text += Colors.RED + "\nSave\n" + Colors.END
        text += "41. Save text(임시파일 Load는 31 활용)\n"
        text += "42. Save a part of df(특정 계정과목 추출 등)\n"        

        text += Colors.RED + "\nGeneral\n" + Colors.END
        text += "90. MANUAL HANDLING - DEBUG\n"
        text += "98. df.info()\n"        
        text += "99. df.head(10)"

        #####
        textMain = Colors.RED + "MAIN MODE : enter '?' to help / 'q' to exit" + Colors.END

        while True:

            print("") #여기다 BREAK를 걸면 디버깅

            print(textMain)
            flag = input(">>")

            match flag:
                case '?': print(text)
                case 'q': print("END"); break
                
                case '11': print("USE VBA...(추후 연동예정)")
                case '12': runConcatText()
                case '13': ConcatTextTest().run()
                case '14': 
                    if isinstance(self.df, pd.DataFrame): self.df = Merger(self.df).run()
                    else: self.df = Merger().run()

                case '21': runReadColumnLength()
                case '22': print("USE mySQL")
                case '23': runTxt2Db()

                case '31': self.df = runImport2Df()
                case '32': self.df = AutoMap().autoMap(self.df)
                case '33':
                    md = Modifier(self.df) #의존성 주입
                    md.run()
                case '34': ReconGL(self.df).run()

                case '41': Saver(self.df).run()
                case '42': SaverPart(self.df).run()                
                case '90':
                    print("DEBUG NOW") #여기다 BREAKPOINT를 걸면 수기 디버깅가능
                case '98': self.df.info()
                case '99': print(self.df.head(10))
                case _: print("Retry"); continue

def run(): 
    spot = Spotlight()
    spot.run()

if __name__=="__main__":
    run()



#에러가 발생하면 반복하는 Decorator
class ErrRetry:
    def __init__(self,func):
        self.func = func
    def __call__(self,*arg,**kwarg):
        while True:
            try:
                return self.func(self, *arg, **kwarg)
            except Exception as e:
                print(e)
                input("RETRY>")        

        # return self.func(*arg, **kwarg) # TO DEBUG

def ErrRetryF(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print(e)
    return wrapper

@ErrRetry
def Test():
    print("a")




import pandas as pd

import spotlight.common.myFileDialog as myfd

from spotlight.common.ErrRetry import ErrRetryF


class AutoMap:

    @ErrRetryF
    def autoMap(self, df:pd.DataFrame)->pd.DataFrame : 

        print("Auto Mapping. Read 매핑파일")
        #filenameImportMap = "ImportMAP.xlsx"
        #filenameImportMap = glob.glob(tgtdir+"/"+filenameImportMap)
        filenameImportMap = myfd.askopenfilename("MAP파일 선택")    
        dfMap = pd.read_excel(filenameImportMap, sheet_name='MAP_GL')

        ## a. MAP 대상 먼저 전처리
        dfMapMap = dfMap[dfMap['방법'] == 'MAP']
        dfTB = pd.DataFrame()
        for i in range(0, dfMapMap.shape[0]):    
            try:
                dfTB[dfMapMap.iloc[i]["tobe"]] = df[dfMapMap.iloc[i]["asis"]]
            except:
                dfTB[dfMapMap.iloc[i]["tobe"]] = df[str(dfMapMap.iloc[i]["asis"])]

        ## b. KEYIN 대상 추가 전처리
        dfMapKeyin = dfMap[dfMap['방법'] == 'KEYIN']

        for i in range(0, dfMapKeyin.shape[0]):        
            dfTB[dfMapKeyin.iloc[i]["tobe"]] = dfMapKeyin.iloc[i]["asis"]


        print("AUTO-MAP Done")
        return dfTB            


import pandas as pd
import glob
import spotlight.common.myFileDialog as myfd
import mmap

# 데이터프레임 컬럼을 trim하는 함수. Call by object reference이므로 return 불필요
def stripColumn(df:pd.DataFrame) -> None:
    df.columns = [str.strip(i) for i in df.columns.to_list()]

def obj2str(df:pd.DataFrame) -> None:
    #df를 받아서, object인 columns을 모두 string으로 변경해줌
    for column in df.columns:    
        if df[column].dtype == 'object':
            df[column] = df[column].astype(str) # Call by Object Refenece이므로 Return 불필요    

def simplecount(filename, encoding):
    lines = 0
    for line in open(filename, encoding=encoding):
        lines += 1
    return lines

def mapcount(filename, encoding) -> int:
    with open(filename, "r+", encoding=encoding) as f: #With clause calls close()
        buf = mmap.mmap(f.fileno(), 0)
        lines = 0
        readline = buf.readline
        while readline():
            lines += 1        
        return lines


import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class DropColumn:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df

    @ErrRetryF
    def run(self, cName:str): #Drop Column
        self.df.drop(columns=[cName], inplace=True) #use inplace
        print("DONE")
        



#231214 : class var로 df를 지정한 후, run method에서 저장방법을 결정함

import pandas as pd
import tqdm

from spotlight.common.protoSelector import ProtoABSSelector
from spotlight.common.ErrRetry import ErrRetryF

@ErrRetryF
class Saver(ProtoABSSelector):

    # df:pd.DataFrame

    # def __init__(self, df:pd.DataFrame):
    #     self.df = df

    def run(self):
        msg = "1. Save to Text\n"
        msg += "2. Save to parquet\n"
        msg += "3. Save to pickle\n"        
        msg += "4. Save to excel\n"        
        print(msg)
        flag = input(">>")
        match(flag):
            case '1': self._saveText()
            case '2': self._saveParquet()
            case '3': self._savePickle()
            case '4': self._saveExcel()            
            case _: print("Wrong enter")

    def _saveText(self) -> None:
        path = input("저장할 Text filename을 지정하세요(기본값 result.tsv)>>") or 'result.tsv'
        encod = input("인코딩? (cp949)>>") or 'utf8'
        sep = input("Seperator? (기본값 \\t)") or '\t'

        pbar = tqdm.tqdm(total=self.df.shape[0], desc='Save')                

        lineStart = 0
        chunksize = 10000000 #천만으로 변경
        flagFirst = True #처음에만 헤더를 넣기 위한 Flag
        while True: #헤더가 추가되는 오류 디버그 231206
            lineEnd = min(lineStart + chunksize, self.df.shape[0])
            length = lineEnd - lineStart
            if flagFirst: self.df.iloc[lineStart:lineEnd, :].to_csv(index=False, sep=sep, encoding=encod, path_or_buf=path, mode='a')            
            else: self.df.iloc[lineStart:lineEnd, :].to_csv(index=False, sep=sep, encoding=encod, path_or_buf=path, mode='a', header=None)
            pbar.update(length)
            lineStart += length

            if flagFirst: flagFirst = False
            if lineEnd >= self.df.shape[0]: break       

        print("save done")
    
    def _saveParquet(self):
        path = input("저장할 Parquet filename을 지정하세요(기본값 result.parquet)>>") or 'result.parquet'      
        self.df.to_parquet(path)
        print("save done")

    def _savePickle(self):
        path = input("저장할 Pickle filename을 지정하세요(기본값 result.pickle)>>") or 'result.pickle'      
        self.df.to_pickle(path)
        print("save done")

    def _saveExcel(self):
        path = input("저장할 Excel filename을 지정하세요(기본값 result.xlsx)>>") or 'result.xlsx'      
        self.df.to_excel(path, index=None)
        print("save done")



import pandas as pd

from spotlight.modify.modify import Modifier
from spotlight.common.colors import Colors

class SetKey(Modifier): #Inhereted Modifier, so initializer has df var.

    cName:list #무조건 List로 받는다.    

    def __init__(self, df:pd.DataFrame = None): #Override
        self.df = df
        self.cName = []

    def run(self, msg:str = "") -> list:
        while True:            
            
            print(Colors.RED+msg+Colors.END)

            print(Colors.CYAN+"현재 KEY :", end='')
            print(self.cName, end='')
            print(Colors.END)
            flag:str = input(Colors.RED+"Add key(1) / End(2)>>"+Colors.END)
            match(flag):
                case '1':
                    cName = self.selectColumn(Colors.RED+"Select Join Column (df1)"+Colors.END, self.df)
                    if cName in self.cName: print("이미 입력한 KEY입니다.")
                    else: self.cName.append(cName)
                case '2':
                    if len(self.cName) == 0: print("아무 KEY도 입력되지 않았습니다.")
                    else: 
                        print("Key 입력을 종료합니다. 현재 KEY : ", end=''); print(self.cName)
                        return self.cName
                case _:
                    print("잘못 입력하였습니다.")
            


import pandas as pd
import tqdm

import spotlight.common.myFileDialog as myfd

class Txt2Df: #50만행씩 READ
    def run(cls, fileName, delimiter : chr = "\t", encoding :str = 'cp949', headerBool : bool = True, chunksize = 500000) -> pd.DataFrame :

        dfCon = pd.DataFrame()    
        totalChunk = 0        

        if headerBool: #헤더가 있다고 받았으면
            headerArg = 0
        else:
            headerArg = None

        for chunk in pd.read_csv(fileName, encoding = encoding, delimiter=delimiter, low_memory=False, header = headerArg, chunksize=chunksize):   #delimiter
            dfCon = pd.concat([dfCon,chunk])
            totalChunk = totalChunk + chunk.shape[0]
            
        #print(fileName,":",totalChunk, "행 imported")        

        return dfCon


#############################
# ForSAP : SAP 추출 data를 위해 별도로 생성한 Class 
# 별도 Class로 분리
#############################
import pandas as pd

class ForSAP:
          
    @classmethod
    def InvertMinus(cls, tgt:str) -> str:
        if len(tgt) <= 1 : return tgt #1글자거나(-) 1글자보다 적으면('') 그냥 바로 반환
        if tgt[-1] == '-':
            tgt = tgt.replace('-','')
            tgt = "-" + tgt
            return tgt
        return tgt

    @classmethod
    def ColumnStrToInt(cls, df:pd.DataFrame, ColumnName:str) -> None:    
    #숫자 뒤에 -를 붙여서 음수가 추출된 경우 앞으로 붙여주는 함수
        try:
            #df[ColumnName] = df[ColumnName].fillna(0).apply(str.strip).apply(lambda x : x.replace(',','') ).apply(lambda x:x.replace('','0')).astype('float64').astype('int64')
            #df[ColumnName] = pd.to_numeric(df[ColumnName].fillna(0).astype('str').apply(str.strip).apply(lambda x : x.replace(',','')).apply(InvertMinus).replace( '[,)]','', regex=True).replace('[(]','-',regex=True))
            tmp = df[ColumnName].fillna(0) #일단 공백을 죽이고
            tmp = tmp.astype('str') #문자로 만든 다음
            tmp = tmp.apply(str.strip) #trim처리하고
            tmp = tmp.apply(lambda x: x.replace(',','')) #쉼표는 없앤다.
            tmp = tmp.replace('^-$', '0', regex=True) #그냥 오직 -는 0으로 바꿔준다.
            # 이후 로직은 위 아래 중 택일해서 돌려아함
            #tmp = tmp.apply(InvertMinus) 
            Flag = input("음수표시가 ()면 1, 마지막글자-면 2, 이외엔 그냥 엔터>>")
            match Flag:
                case '1':
                    tmp = tmp.replace( '[,)]','', regex=True).replace('[(]','-',regex=True) #()를 마이너스로 바꿔주는 함수        
                case '2':
                    tmp = tmp.apply(cls.InvertMinus)
                case _:
                    print("선택하지 않았습니다.")                

            tmp = pd.to_numeric(tmp, errors='coerce') #에러는 0으로..
            df[ColumnName] = tmp

        except Exception as e:
            print(e)        

       ## 전표금액 가공부2. SAP은 100을 곱해줘야 한다.
    @classmethod
    def Multiple100(cls, df:pd.DataFrame, columnName:str):
        df[columnName] = df[columnName] * 100


########################################################################################################################################################################



    
import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class DropDuplicate:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df

    @ErrRetryF
    def run(self): #Drop Column
        print("drop_duplicates() with imported df")                
        cntBefore = self.df.shape[0]
        self.df.drop_duplicates(inplace=True)
        cntAfter = self.df.shape[0]
        print("제거된 중복행수 = ", cntBefore - cntAfter)
        print("DONE")
    



#For inheritance
#Class var : df
#Class method : selectColumn

from abc import ABCMeta
from abc import abstractmethod

import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class ProtoSelector:
    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame = None): #의존성 주입
        self.df = df

    @ErrRetryF
    def selectColumn(self, msg:str = "Select column", df:pd.DataFrame = None) -> str: #DF Injection은 선택. 기본값은 self.df 사용
        print('\n'+msg+'\n')
        if not isinstance(df,pd.DataFrame): df = self.df # USE Self.df when no injected df
        df.info()
        num = input(">>")
        num = int(num) 
        cName = df.columns[num] #선택한 번호에 해당하는 컬럼명을 반환한다.
        return cName
    
class ProtoABSSelector(ProtoSelector, metaclass=ABCMeta):
    @abstractmethod
    def run(self):
        pass






import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class FillNA:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df

    @ErrRetryF
    def run(self, cName:str): #fillna(0)

        flag = input("dtype이 object 또는 str입니까? (기본값 Y)>>")
        if flag:
            self.df[cName] = self.df[cName].fillna('0')
        else:
            self.df[cName] = self.df[cName].fillna(0)
                    
        print("DONE")
        



import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

#SAP 전표 다음 유형 2가지를 음수로 바꿔주는 클래스
# 유형1 : (100)
# 유형2 : 100-
class ToMinus:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df
    
    def run(self, cName:str):
        flag = input("1. (), 2. 후위- >>")
        match flag:
            case '1': self.runBracket(cName)
            case '2': self.runPostfix(cName)
            case _: print("아무것도 입력하지 않았습니다.")
    
    @ErrRetryF
    def runBracket(self, cName:str): #()

        self.df[cName] = self.df[cName].astype('str')

        self.df[cName] = self.df[cName].replace('[)]','',regex=True)
        self.df[cName] = self.df[cName].replace('[()]','-',regex=True)

        self.df[cName] = self.df[cName].astype('float64')

        print("DONE")

    @ErrRetryF
    def runPostfix(self, cName:str): #()
        
        self.df[cName] = self.df[cName].astype('str')    

        self.df[cName] = self.df[cName].apply(self.InvertMinus)

        self.df[cName] = self.df[cName].astype('float64')
                
        print("DONE")

    def InvertMinus(self, tgt:str) -> str:
        if len(tgt) <= 1 : return tgt #1글자거나(-) 1글자보다 적으면('') 그냥 바로 반환
        if tgt[-1] == '-':
            tgt = tgt.replace('-','')
            tgt = "-" + tgt
            return tgt
        return tgt


import tqdm
import pandas as pd
import pymysql
import sqlalchemy
pymysql.install_as_MySQLdb()
import MySQLdb    
import pyodbc

import spotlight.common.myFileDialog as myfd

class Df2Db:
    
    engine : sqlalchemy.Engine #class variable
    
    def __init__(cls, engine:sqlalchemy.Engine): #생성자 매개변수로 sqlalchemy.engine을 받는다
        cls.engine = engine 
        print("Engine connected")    
    
    def insert(cls, df : pd.DataFrame, tableName : str, chunksize=1000000): #insert dataframe to db # chunksize : 1백만. 이거보다 적으면 느려짐
        tgtNo = df.shape[0] #rows to insert        
        #pbar = tqdm.tqdm(total=tgtNo , desc="작업대상행수")

        startNo = 0
        endFlag = True
        resultInsertAcc = 0                

        while endFlag:        
            endNo = startNo + chunksize    
            if (endNo > tgtNo):
                endNo = tgtNo
                endFlag = False #이번 루프를 마지막으로 종료시키기
            dfTmp = df.iloc[startNo:endNo,:]   #df1 : 입력할 chunk
            #tmp = pbar.update(endNo-startNo)            
            resultInsert = dfTmp.to_sql(name=tableName, con=cls.engine, index=False, if_exists="append")
            #print(startNo,"/",endNo,"/ Insert 완료")
            startNo = startNo + chunksize
            resultInsertAcc = resultInsertAcc + resultInsert            
        #pbar.close()   
        print(resultInsertAcc,"행 Inserted")


#DATAFRAME MERGE
import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF
from spotlight.common.protoSelector import ProtoABSSelector
from spotlight.modify.modify import Modifier
from spotlight.import2df.import2df import runImport2Df
from spotlight.save.save import Saver
from spotlight.common.colors import Colors
from spotlight.merge.setkey import SetKey

class Merger(ProtoABSSelector): #Inherit Modifier to use 'selectColumn'
    dfA:pd.DataFrame
    dfB:pd.DataFrame
    dfJoin:pd.DataFrame
    #상속받은 df변수를 dfNow 성격으로 재활용함    
    flag:str #A | B | Join | NONE, 기본값 NOT
    
    def __init__(self, df:pd.DataFrame = None): #OVERRIDE
        if isinstance(df, pd.DataFrame): self.dfA = df
        else: pass        
        self.flag = "NONE"
    
    @ErrRetryF    
    def run(self) -> pd.DataFrame:

        text =  Colors.RED + "#"*10+"\n" + Colors.END
        text += "11. Set DF1\n"    
        text += "12. Set DF2\n"
        text += "21. MODIFY\n"
        text += "31. Join(Merge) DF1 & DF2\n"
        text += "41. Export DFJoin to textfile\n"
        text += "42. Connect DFJoin to Main Mode (이어서 바로 작업할 수 있게)\n\n"
        text += "50. Select DF(1,2,Join 중에) => MODIFY, Export, Connect와 연계됨\n\n"
        text += Colors.RED + "General\n" + Colors.END
        text += "90. MANUAL HANDLING - DEBUG\n"
        text += "91. df1.info()\n"        
        text += "92. df1.head(10)\n"        
        text += "93. df2.info()\n"        
        text += "94. df2.head(10)\n"
        text += "95. dfJoin.info()\n"        
        text += "96. dfJoin.head(10)\n"

        #####
        textMain = Colors.RED + "JOIN MODE : enter '?' to help / 'q' to exit" + Colors.END
        if self.flag != "NONE": textMain += Colors.RED + "Selected df:" + self.flag + Colors.END

        while True:

            print(textMain)
            flag = input(">>")

            match flag:
                case '?': print(text)
                case 'q': print("END to Main mode"); break                
                
                case '11': self.dfA = runImport2Df("Join할 파일을 선택하세요(1)")
                case '12': self.dfB = runImport2Df("Join할 파일을 선택하세요(2)")

                case '21': self.enterModify()
                
                case '31': 
                    self._merge()
                    self._selectDF('Join')

                case '41': Saver(self.df).run()
                case '42': print("Return to main mode with selected df"); return self.df

                case '90':
                    print("DEBUG NOW") #여기다 BREAKPOINT를 걸면 수기 디버깅가능
                case '91': self.dfA.info()
                case '92': print(self.dfA.head(10))

                case '93': self.dfB.info()
                case '94': print(self.dfB.head(10))

                case '95': self.dfJoin.info()
                case '96': print(self.dfJoin.head(10))

                case _: print("Retry"); continue

    def _merge(self) -> None:        
        #cNameJoinA:str = self.selectColumn("Select Join Column (df1)", self.dfA)
        #cNameJoinB:str = self.selectColumn("Select Join Column (df2)", self.dfB)
        cNameJoinA:list = SetKey(self.dfA).run("set Key : dfA")
        cNameJoinB:list = SetKey(self.dfB).run("set Key : dfA")        
        
        if not self.check(cNameJoinA, cNameJoinB): print("중단합니다."); return #VALIDATE

        howMerge = input("Select join method(기본값 : inner)>>") or 'inner'

        if (input("USE Indicator? (기본값 : N)") or 'N') == 'Y': self.dfJoin = self.dfA.merge(right=self.dfB, left_on=cNameJoinA, right_on=cNameJoinB, how=howMerge, indicator=True)
        else: self.dfJoin = self.dfA.merge(right=self.dfB, left_on=cNameJoinA, right_on=cNameJoinB, how=howMerge)

        print("dfA 행수 : ",self.dfA.shape[0])
        print("dfB 행수 : ",self.dfB.shape[0])
        print("dfJoin 행수 : ",self.dfJoin.shape[0])
        #print("검증 : ", self.dfJoin.shape[0] == self.dfA.shape[0] + self.dfB.shape[0])

        print("DONE")
        
    def check(self, cNameJoinA:list, cNameJoinB:list) -> bool:
        print("신뢰성 있는 JOIN을 위해서는 LEFT-KEY와 RIGHT-KEY가 PRIMARY KEY여야 합니다. (이외의 경우 중복값 발생)")
        print("(i.e. Left outer join 시에는 Right Table의 KEY가 PK여야만 함 (LEFT는 상관없음))")
        print(Colors.CYAN+"테스트를 실시합니다."+Colors.END)
        print("dfA>>")
        bA = self._checkDetail(self.dfA, cNameJoinA)
        print("dfB>>")
        bB = self._checkDetail(self.dfB, cNameJoinB)

        if bA and bB: print("검증 PASS. 진행합니다."); return True
        else: #검증에 실패했을 때
            flag = input("검증실패. 그럼에도 불구하고 진행하겠습니까?\n 이 경우 join으로 record count가 늘어나 신뢰성을 보장할 수 없습니다.(기본값 N)>>") or 'N'
            match(flag):
                case 'Y': return True
                case 'N': return False
                case _: print("잘못된 값 입력. 종료합니다."); return False

    #각 DF별 체크하는 메서드
    def _checkDetail(self, df:pd.DataFrame, cName:list) -> bool:
        dfValueCount = df[cName].value_counts()
        result = dfValueCount[(dfValueCount != 1)]

        if result.shape[0] == 0: print("중복되는 KEY가 없습니다."); return True
        elif result.shape[0] != 0: print("중복이 있는 KEY>>"); print(result); return False   

    def enterModify(self):
        print("entering modify mode to modify df(selected)...")
        flag:str = input("A or B or Join>>")
        dfTmp:pd.DataFrame
        match(flag):
            case 'A': dfTmp = self.dfA
            case 'B': dfTmp = self.dfB
            case 'Join': dfTmp = self.dfJoin
            case _: print("잘못된 입력입니다."); return
        Modifier(dfTmp).run()        

    def _selectDF(self, flag:str = ''):
        print("Select DF...")
        if flag in ['A','B','Join']: self.flag = flag
        else: self.flag:str = input("A or B or Join>>")
        match(self.flag):
            case 'A': 
                if isinstance(self.dfA,pd.DataFrame): self.df = self.dfA
                else: print("dfA not loaded yet")
            case 'B':
                if isinstance(self.dfB,pd.DataFrame): self.df = self.dfB
                else: print("dfB not loaded yet")
            case 'Join':
                if isinstance(self.dfJoin,pd.DataFrame): self.df = self.dfJoin
                else: print("dfJoin not loaded yet")            
            case _: print("잘못된 입력입니다."); return
        print("현재 선택된 df:"+self.flag)
        


# 231213 : 하위폴더 순환하도록 구현
# 231213 : 마지막에 LF를 포함하지 않는 경우 강제로 추가하여 개행하도록 수정

# 전역선언부
import os
import glob
import tqdm
import spotlight.common.myFileDialog as myfd

# 함수부
class ConcatText:
    liTgt:list    
    filenameNew:str
    encodingOld:str
    encodingNew:str
    bHeader:bool

    def run(cls): #HANDLER        
        cls.getFiles()
        cls.filenameNew = input("new file name(result.txt)>>") or 'result.txt'                
        cls.encodingOld = input("encodingOld(cp949)>>") or 'cp949'
        cls.encodingNew = input("encodingNew(utf8)>>") or 'utf8'

        flag = input("헤더가 포함되어 있습니까?(기본값 Y). Y인 경우 첫번째 파일만 포함함)>>") or 'Y'
        if flag == 'Y': cls.bHeader = True
        else: cls.bHeader = False

        cls.concat(cls.bHeader)
        print("DONE")

    #231213: 하위폴더 순환하도록 변경
    def getFiles(cls, msg:str = "합칠 텍스트 파일이 있는 경로명(하위폴더 포함)"): 
        path = myfd.askdirectory(msg)        
        ext = input("확장자(tsv)>>") or 'tsv'        
        strContain = input("특정 문자열을 포함하는 파일만 포함하고자 하는 경우 입력하세요.(기본값 None. None인 경우 모두 포함함)>>") or None #포함할 문자열
        #cls.liTgt = glob.glob(path + '/*.'+ext)    #result.txt 생성 앞에 해야함
        cls.liTgt = []
        for dirpath, dirname, filenames in os.walk(path):
            for file in filenames:
                path = "%s/%s"%(dirpath,file)
                path = path.replace('\\','/').replace('//','/')

                if strContain != None:
                    if (path.split(".")[-1] == ext) and (strContain in path): cls.liTgt.append(path)
                elif strContain == None:
                    if path.split(".")[-1] == ext: cls.liTgt.append(path)

    def concat(cls, bHeader:bool): #bHeader가 True면 헤더 포함
        # 합산파일 생성
        fileNew = open(cls.filenameNew,'wt', encoding=cls.encodingNew)
        fileNew.close()

        # 파일 하나씩 append        
        fileNew = open(cls.filenameNew,'at', encoding=cls.encodingNew)        
        print("총",len(cls.liTgt),"개")
        pbar = tqdm.tqdm(total=len(cls.liTgt) , desc="순환")
        totalj = 0
        i = 0 #i = File
        for file in cls.liTgt:    
            bContainLF = cls.checkLF(file, cls.encodingOld) #파일을 열 때, 해당 파일 마지막에 LF를 포함하는지 확인한다.           

            j = 0 #j = Line           
            fileOld = open(file, 'rt', encoding=cls.encodingOld)
            for line in fileOld:                
                if bHeader and i!=0 and j==0: pass #헤더가 True이고, 첫번째 파일이 아닌 다른 파일인 경우, 첫번째 줄이면 생략함
                else: tmp = fileNew.write(line) #to speed up, set return
                j+=1
                
            fileOld.close(); i+=1
            if not bContainLF: fileNew.write('\n'); print("ADDED LF AFTER :", file) #when not contain LF, add LR (NOT CRLF)
            totalj += j
            msg = str(file)+":"+str(totalj)
            pbar.desc = msg; pbar.update(1) #pbar update

        fileNew.close()
        pbar.close()

    def checkLF(self, filename:str, encoding:str) -> bool:
        f = open(filename, mode='rb')
        f.seek(-1, os.SEEK_END)
        tmp = f.read(1) #read 1 byte)
        f.close()

        return str(tmp, encoding=encoding) == '\n' #마지막 Byte가 LF인지를 확인하여 반환         

def runConcatText():
    ConcatText().run()

if(__name__ == "__main__"):
    runConcatText()


import pandas as pd
import tqdm

import spotlight.common.myFileDialog as myfd
from spotlight.common.ErrRetry import ErrRetry
from spotlight.common.common import mapcount

class Import2Df:    
    def run(self, msg:str = "") -> pd.DataFrame:
        
        help = "1. import txt\n"
        help += "2. import parquet\n"
        help += "3. import pickle\n"
        help += "4. import Excel\n"        
        print(help)
        flag = input(">>")
        match(flag):
            case '1': return self._importTxt(msg)
            case '2': return self._importParquet(msg)
            case '3': return self._importPickle(msg)
            case '4': return self._importExcel(msg)            
            case _: print("Wrong"); pass

    @ErrRetry
    def _importTxt(self, msg:str = "Text 파일을 선택하세요.") -> pd.DataFrame:       
        path = myfd.askopenfilename(msg)
        sep = input("Seperator? (기본값 \\t)>>") or '\t'
        encod = input("인코딩? (cp949)>>") or 'cp949'        
        
        cnt = mapcount(path,encod) -1 #Count Check _ Column 때문에 1을 빼야 함
        pbar = tqdm.tqdm(total=cnt, desc='Read')        

        df = pd.DataFrame()
        chunksize = 10000000 #천만
        dfReader = pd.read_csv(path, sep=sep, encoding=encod, low_memory=False, chunksize=chunksize)
        for count, chunk in enumerate(dfReader):
            df = pd.concat([df, chunk])
            pbar.update(chunk.shape[0])
        pbar.close()
        return df

    def _importParquet(self, msg:str = "Select parquet") -> pd.DataFrame:
        path = myfd.askopenfilename(msg)
        return pd.read_parquet(path)

    def _importPickle(self, msg:str = "Select pickle") -> pd.DataFrame:
        path = myfd.askopenfilename(msg)
        return pd.read_pickle(path)

    def _importExcel(self, msg:str = "Select Excel") -> pd.DataFrame:
        path = myfd.askopenfilename(msg)
        return pd.read_excel(path)

def runImport2Df(msg:str = "") -> pd.DataFrame:
    return Import2Df().run(msg)

if __name__=='__main__':
    runImport2Df()


import pandas as pd

#from spotlight.main import Spotlight
from spotlight.common.ErrRetry import ErrRetryF
from spotlight.common.protoSelector import ProtoSelector, ProtoABSSelector
from spotlight.modify.replace import Replacer
from spotlight.modify.applyDC import ApplyDC
from spotlight.modify.minus import ToMinus
from spotlight.modify.m100 import M100
from spotlight.modify.createUnsigned import CreateUnsigned
from spotlight.modify.fillna import FillNA
from spotlight.modify.automanual import AutoManual
from spotlight.modify.dropColumn import DropColumn
from spotlight.modify.dropDuplicate import DropDuplicate
from spotlight.common.colors import Colors

class Modifier(ProtoABSSelector):

    #df:pd.DataFrame

    # def __init__(self, df:pd.DataFrame = None): #의존성 주입
    #     self.df = df

    def run(self) -> None:
        text =  "#"*10+"\n"
        text += Colors.RED + "MODIFY MODE\n" + Colors.END
        text += Colors.RED + "전처리_전표금액\n" + Colors.END
        text += "11. replace string (Kill comma, space, regex etc.)\n" #
        text += "12. Apply DC (차변은 +, 대변은 - 처리)\n" #차대구분에 따라 (-)처리 : 변경컬럼선택 / 차대컬럼선택 / 차변구분자 선택 / 시행        
        text += "13. To Minus () or - : 문자열 () 또는 후위-인 경우 마이너스로 변환\n" #마이너스전환 : ()인 경우
        text += "14. Multiple 100  : 곱하기 100\n" #*100 : 변경컬럼선택 / 시행
        text += "15. FROM 전표금액 TO 차대금액  : signed 전표금액에서 unsigned 차변/대변을 생성한다.\n" #전표금액을 차대금액으로 뿌리기 : 전표기능/ 전표표시 선택 / (+)(-)로 자동 뿌림
        text += "16. FILLNA(0)  : N/A를 0으로 채운다.\n" 
        text += "17. 자동수동 : 특정 컬럼값(전표성격, 사용자 등)이 특정 문자열(복수 가능)을 포함하는 행을 A로 지정\n"
        text += "\n"
        text += Colors.RED + "기타 전처리\n" + Colors.END
        text += "21. drop a column\n"
        text += "22. drop duplicate\n"
        text += "23. Change column name\n"
        text += "24. Change column datatype\n"
        text += "\n"
        text += Colors.RED + "General\n" + Colors.END
        text += "90. DEBUG MODE\n"
        text += "98. df.info()\n"
        text += "99. df.head(10)\n"
        #####
        textMain = Colors.RED + "MODIFY MODE : enter '?' to help / 'q' to exit" + Colors.END

        while True:
            print(textMain)
            flag = input(">>")            
            match flag:
                case '?': print(text)
                case 'q': print("MODIFY MODE END"); break
                case '11': #콤마 없애기 : 변경컬럼선택 / 시행
                    cName = self.selectColumn() #컬럼명 추출
                    Replacer(self.df).run(cName) #DF랑 컬럼명을 던진다

                case '12': #콤마 없애기 : 변경컬럼선택 / 시행
                    cName1 = self.selectColumn("변경할 컬럼을 선택하세요") #컬럼명 추출
                    cName2 = self.selectColumn("차대구분자컬럼을 선택하세요(D/C, S/H 등)") #컬럼명 추출
                    ApplyDC(self.df).run(cName1, cName2)                     
                
                case '13':
                    cName = self.selectColumn("변경할 컬럼을 선택하세요") #컬럼명 추출
                    ToMinus(self.df).run(cName)

                case '14':
                    cName = self.selectColumn("변경할 컬럼을 선택하세요") #컬럼명 추출
                    M100(self.df).run(cName)

                case '15':                    
                    cNameSigned = self.selectColumn("select signed amount (From)") #컬럼명 추출
                    cNameUnsignedD = self.selectColumn("select Unsigned Debit (To Create)") #컬럼명 추출
                    cNameUnsignedC = self.selectColumn("select Unsigned Credit (To Create)") #컬럼명 추출
                    CreateUnsigned(self.df).run(cNameSigned, cNameUnsignedD, cNameUnsignedC)

                case '16':
                    cName = self.selectColumn("변경할 컬럼을 선택하세요") #컬럼명 추출
                    FillNA(self.df).run(cName)

                case '17':
                    cNameBase = self.selectColumn("자동수동 구분 기준값을 포함하는 컬럼을 선택하세요(성격 또는 사용자 등)") #컬럼명 추출
                    cNameAM = self.selectColumn("자동수동 컬럼을 선택하세요") #컬럼명 추출
                    AutoManual(self.df).run(cNameBase, cNameAM)

                case '21':
                    cName = self.selectColumn("Drop할 Column을 선택하세요") #컬럼명 추출
                    DropColumn(self.df).run(cName)
                case '22': DropDuplicate(self.df).run()
                case '23': self._changeColumn()
                case '24': self._changeColumnDtype()

                case '90':
                    print("DEBUG NOW") #여기다 BREAKPOINT를 걸면 수기 디버깅가능
                case '98': self.df.info()
                case '99': print(self.df.head(10))

                case _: print("Retry"); continue

    # @ErrRetryF
    # def selectColumn(self, msg:str = "Select column", df:pd.DataFrame = None) -> str: #DF Injection은 선택. 기본값은 self.df 사용
    #     print('\n'+msg+'\n')
    #     if not isinstance(df,pd.DataFrame): df = self.df # USE Self.df when no injected df
    #     df.info()
    #     num = input(">>")
    #     num = int(num) 
    #     cName = df.columns[num] #선택한 번호에 해당하는 컬럼명을 반환한다.
    #     return cName

    @ErrRetryF
    def _changeColumn(self) -> None: 
        cNameBefore = self.selectColumn("변경하고자 하는 컬럼을 선택하세요")        
        cNameAfter = input("변경 후 컬럼명을 입력>>")
        self.df.rename(columns={cNameBefore:cNameAfter}, inplace=True)
        print("DONE")

    @ErrRetryF
    def _changeColumnDtype(self) -> None: 
        cName = self.selectColumn("변경하고자 하는 컬럼을 선택하세요")        
        cDtype = input("변경하고자 하는 자료형을 입력(i.e. string, float64, int64 등)>>")        
        self.df[cName] = self.df[cName].astype(cDtype)
        print("DONE")


# 에프알엘코리아
# tsv 2 df # Test 용. df2db로 대체
#
# 특이사항
# 각 파일 앞 뒤 문자열 삭제 (앞 : 제목, 뒤 : 합계)
# 파일 내 double-quotes 삭제 필요 (EMeditor 활용

#S가 차변 #H가 대변 #세차

# 1. 데이터프레임으로 읽기
del myfd

from mylib import myFileDialog as myfd
path = myfd.askopenfilename()

import pandas as pd
df= pd.read_csv(path,sep="\t", low_memory=False)

df= pd.read_excel(path)

df.shape

# 컬럼명 strip()

tmp = df.columns.to_list()
tmplist = []
for i in tmp:
    tmplist.append(i.strip())
df.columns = tmplist

# 숫자로 만들기 - 거래통화
df['Amount'] = df['Amount'].str.replace(",","")
df['Amount'] = pd.to_numeric(df['Amount'])

# 숫자로 만들기 - 표시통화
df['Loc.curr.amount'] = df['Loc.curr.amount'].str.replace(",","")
df['Loc.curr.amount'] = pd.to_numeric(df['Loc.curr.amount'])

tmp1 = df.groupby('D/C')['Loc.curr.amount'].sum()
tmp1
tmp2 = df.groupby('D/C')['Amount'].sum()
tmp2

df.groupby(['G/L','D/C'])['Loc.curr.amount'].sum()

tmp = df['Unnamed: 29'].drop_duplicates()


path
#컬럼설계
for column in df:
    print(column,"->", df[column].astype(str).str.len().max())

df = df.drop_duplicates()

#문제 없어 보임


###
#메모. 2월은 그냥 따로 파이썬으로 읽어서 넣으면 된다.

string = "		1004	7300012425		2023	SC	2022/10/01	2022/10/01	    02	2022/11/02	12:11:38	0000/00/00	0000/00/00	PI-USER			[점포07]교통비: 보안택 미제거 고객 자택방	KRW			001	S		            5,000	KRW	            5,000	KRW	[점포07]교통비: 보안택 미제거 고객 자택방문	 택시기사님과 커뮤니케이션 미스로 본인		1004	S001135	75030KR001	S001135			0001	PCS-CSS"

string2="		1004	7300012425		2023	SC	2022/10/01	2022/10/01	    02	2022/11/02	12:11:38	0000/00/00	0000/00/00	PI-USER			[점포07]교통비: 보안택 미제거 고객 자택방	KRW			002	H		            5,000	KRW	            5,000	KRW	[점포07]교통비: 보안택 미제거 고객 자택방문	 택시기사님과 커뮤니케이션 미스로 본인		1004		10020KR999	S001135			0001	PCS-CSS"
string.count("\t")

string2.count("\t")

string3="		1004	7300012426		2023	SC	2022/10/12	2022/10/12	    02	2022/11/02	12:11:38	0000/00/00	0000/00/00	PI-USER			[점포07]교통비: 고객 클레임 보안택 미제거	KRW			001	S		            5,400	KRW	            5,400	KRW	[점포07]교통비: 고객 클레임 보안택 미제거 건 (매장 ->자택)			1004	S001176	75030KR001	S001176			0036	PCS-CSS"

string3.count("\t")


import pandas as pd

from spotlight.common.ErrRetry import ErrRetryF

class M100:

    df:pd.DataFrame

    def __init__(self, df:pd.DataFrame):
        self.df = df

    @ErrRetryF
    def run(self, cName:str): #100을 곱해준다

        self.df[cName] = self.df[cName].astype('float64')

        self.df[cName] = self.df[cName] * 100

        print("DONE")
        
